from functools import wraps
from app import app
from app.checker import Checker
import os
from flask import request, render_template, jsonify, Response
from werkzeug.exceptions import abort
from werkzeug.utils import secure_filename
import tempfile

__author__ = 'mirek'


def token_required(f):
    """Checks, whether the valid private-token is provided in request"""
    @wraps(f)
    def decorator(*args, **kwargs):
        # TODO check must be based on db
        if request.headers.get('Private-Token') is None:
            abort(401)
        print("=x" * 50)
        return f(*args, **kwargs)
    return decorator

def check_extension(filename, extension):
    """
    Checks, if filename has required extension.
    """
    return '.' in filename and \
           filename.rsplit('.', 1)[1].upper() == extension.upper()



@app.route("/", methods=['POST'])
@token_required
def check_package():
    """
    package checker
    """

    # check if there are necessary files
    if 'template' not in request.files or 'package' not in request.files:
        abort(400)

    # check the files first
    template = request.files['template']
    package = request.files['package']

    if(not check_extension(template.filename, 'xml') or
       not check_extension(package.filename, 'zip')):
        return 'Unsupported Files'

    tpl_path = None
    pkg_path = None
    temp = None

    try:
        # save files
        temp = tempfile.mkdtemp()
        tpl_path = os.path.join(temp, secure_filename(template.filename))
        pkg_path = os.path.join(temp, secure_filename(package.filename))
        request.files['template'].save(tpl_path)
        request.files['package'].save(pkg_path)

        checker = Checker(pkg_path, tpl_path)
        checker.validate()

        accept = request.headers.get('accept').lower()

        # xml
        if accept == 'text/xml':
            template = render_template(
                'xml-result.xml',
                result=checker.get_result(),
                metadata=checker.get_metadata())
            return Response(template, content_type='text/xml; charset=utf-8')

        # html
        elif accept == 'text/html':
            return render_template(
                'html-result.html',
                result=checker.get_result(),
                metadata=checker.get_metadata())

        # txt
        elif accept == 'text/plain':
            template = render_template(
                'txt-result.txt',
                result=checker.get_result(),
                metadata=checker.get_metadata())
            return Response(template, content_type='text/plain; charset=utf-8')

        # json
        elif accept in ('application/json', '*/*'):
            result = checker.get_result()
            metadata = checker.get_metadata()
            metadata['start'] = str(metadata['start'])
            metadata['end'] = str(metadata['end'])
            metadata['duration'] = str(metadata['duration'])
            result['metadata'] = metadata
            return jsonify(result)

        # other types
        else:
            abort(400)

    finally:
        if temp is not None:
            os.unlink(tpl_path)
            os.unlink(pkg_path)
            os.rmdir(temp)